//****************************************************************************
//
// BITSTREAM.H - Contains global definitions and function prototypes for the
//               buffer bitstream.
//
// Copyright (c) 2000 START GROUP.
//
//****************************************************************************

#ifndef _bitstream_h_
#define _bitstream_h_

/* Fast get bits Declarations */
#define FAST_GETBITS_DEFINE    register unsigned int *Ptr, Idx, Val;

#define FAST_GETBITS_INIT(BS)  { Ptr=BS->bufptr;        \
                                 Idx=BS->bitidx;        \
                                 Val=*Ptr<<Idx; }

#define FAST_GETBITS_FIN(BS)   { BS->bufptr=Ptr;        \
                                 BS->bitidx=Idx;        }

#define FAST_IGNORT(N)         { Idx += N;              \
                                 Ptr += Idx >> 5;       \
                                 Idx &= 0x1f;           }

#define FAST_GET(DEST,N)       { register unsigned int A, B, C; \
                                 C=N;                           \
                                 asm("RSB   A,C,#32;            \
                                      MOV   B,Val,LSR A;        \
                                      SUBS  A,A,Idx;            \
                                      LDRLT Val,[Ptr,#4]!;      \
                                      ADDLT A,A,#32;            \
                                      ORRLT B,B,Val,LSR A;      \
                                      RSB   Idx,A,#32;          \
                                      MOVLT Val,Val,LSL Idx;    \
                                      MOVGE Val,Val,LSL C");    \
                                 DEST = B; }

#define FAST_PREVIEW(DEST,N)   { register unsigned int A, B, C; \
                                 A=N;                           \
                                 asm{"RSB   A,A,#32;            \
                                      MOV   B,Val,LSR A;        \
                                      SUBS  A,A,Idx;            \
                                      LDRLT C,[Ptr,#4];         \
                                      ADDLT A,A,#32;            \
                                      ORRLT B,B,C,LSR A");      \
                                 DEST = B; }

/* Type Definitions */
typedef struct {
    unsigned int   *bufptr;
    unsigned int    bitidx;
} Bit_stream_struc;

#ifdef __cplusplus
extern "C"
{
#endif /* __cplusplus */

/* Function Prototype Declarations */
extern unsigned int get_bits(Bit_stream_struc *, int);
extern unsigned int preview_bits(Bit_stream_struc *, int);
extern void ignore_bits(Bit_stream_struc *, int);
extern void put_bits(Bit_stream_struc *, unsigned int, int);
extern void flush_bits(Bit_stream_struc *);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
